#
# Computation and graphing of CIs for decoy/attraction experiments
# -- Somehow generic helper functions
# Pierre Dragicevic, Jan 2016
#

rm(list = ls()) # clean up R's memory
library(PropCIs)
library(ggplot2)
library(boot)

#
# Function for computing a difference between two independent proportions
# and its 95% confidence interval
#
diffpropCI <- function(x1, n1, x2, n2) {
  diffprop <- x1/n1 - x2/n2
  # For a comparison of different CI methods, see our google doc
  # https://docs.google.com/document/d/1RsX6MxkvQ47gfMQlwqnyBai1MBNBnHFad_LdX3U3ZwI/edit
  # Here we're using the "Score interval for difference of proportions and
  # independent samples (p1 - p2)"
  CI <- diffscoreci(x1 = x1, n1 = n1, x2 = x2, n2 = n2, conf.level = 0.95)
  # require('ExactCIdiff')
  #CI <- BinomCI(x = x1, n1 = n1, y = x2, n2 = n2, conf.level = 0.95, CItype='Two.sided')
  # CI <- wald2ci(x1 = x1, n1 = n1, x2 = x2, n2 = n2, conf.level = 0.95, adjust='AC')
  c(diffprop, CI$conf.int[1], CI$conf.int[2])
}

#
# Function for computing the mean of non-normally distributed data and its
# 95% (bootstrap) confidence interval
#
meanCI <- function(observations) {
  samplemean <- function(x, d) {return(mean(x[d]))}
  mean <- samplemean(observations)
  # Compute the bootstrap confidence interval
  set.seed(0) # make deterministic
  bootstrap_samples <- boot(data = observations, statistic = samplemean, R = 5000)
  bootci <- boot.ci(bootstrap_samples, type = "bca")
  c(mean,  bootci$bca[4], bootci$bca[5])
}

#
# Pad a string with white spaces. Use NA for no padding and a negative width to pad right (align left).
#
pad <- function(s, width) {
  if (is.na(width))
    s
  else
    sprintf(paste('%', width, 's', sep=''), s)
}

#
# Round a number for textual presentation
# 
format_number <- function(n, sigdigits = NA, width = NA, percent = FALSE) {
  
  if (percent)
    n <- n * 100
  suffix <- ifelse(percent, '%', '')
  
  if (!is.na(sigdigits)) {
    s <- format_number_(n, width, sigdigits = sigdigits)
  } else {
    # When sigdigits = NA, we use what seems to be a reasonable manual heuristic for deciding on
    # the number of significant digits in our case (decoy study)
    if (abs(n) >= 10)
      s <- format_number_(n, sigdigits = 2, width = width)
    else if (abs(n) >= 2)
      s <- format_number_(n, sigdigits = 1, width = width)
    else if (abs(n) >= 1)
      s <- format_number_(n, sigdigits = 2, width = width)
    else
      s <- format_number_(n, sigdigits = 1, width = width)
  }
  
  paste(s, suffix, sep='')
}

#
# Internal function for rounding a number for textual presentation
#
format_number_ <- function(n, sigdigits, width = NA) {
  # I finally seem to have found a pretty formatting code that actually works!
  pad(formatC(signif(n, digits=sigdigits), digits=sigdigits, format="fg", flag="#", big.mark = " "), width)
}

#
# Present a number and its CI
#
format_ci <- function(x, sigdigits = NA, width = NA, percent = FALSE) {
  nwidth <- floor((width - 8) / 3)
  pad(paste(
    format_number(x[1], sigdigits = sigdigits, width = nwidth, percent = percent),
    ', CI[',
    format_number(x[2], sigdigits = sigdigits, width = nwidth, percent = percent),
    ',',
    format_number(x[3], sigdigits = sigdigits, width = nwidth, percent = percent),
    ']',
    sep=''), width = width)
}
